#include "playlistcontroller.h"
#include "debug.h"
#include "configuration.h"

PlayListController::PlayListController(QObject *parent) :
    QObject(parent)
{
    DGUARDMETHOD;

    DPRINTF("Initialize MediaCache class...");
    mediaDatabase_ = new MediaDatabase(qGlobalConfig.databaseFilename());
    mediaDatabase_->load();
    DPRINTF("Done");

    playbackController_ = new PlaybackController(this);
}

PlayListController::~PlayListController()
{
    qDeleteAll(playLists_); // will also delete all aspects...

    mediaDatabase_->save();
    delete mediaDatabase_;
}

void PlayListController::playListDestroyed()
{
    PlayList *signalSource = static_cast<PlayList *>(sender());

    // Remove all traces of this playlist
    playLists_.removeOne(signalSource);
    QString name = playListMap_.key(signalSource, QString::null);
    if (!name.isNull())
        playListMap_.remove(name);
}

void PlayListController::playListAspectDestroyed()
{
    PlayListAspect *signalSource = static_cast<PlayListAspect *>(sender());

    // Remove all aspects for this playlist
    playListAspects_.removeOne(signalSource);
    QString name = playListAspectMap_.key(signalSource, QString::null);
    if (!name.isNull())
        playListAspectMap_.remove(name);
}

PlayList *PlayListController::createPlayList(const QString &name)
{
    if (!playListMap_.contains(name))
    {
        PlayList *result = new PlayList(mediaDatabase(), name);

        connect(result, SIGNAL(destroyed()), this, SLOT(playListDestroyed()));

        playLists_.append(result);
        playListMap_.insert(name, result);
        return result;
    }
    else
        return NULL;
}

bool PlayListController::deletePlayList(const QString &name)
{
    PlayList *playList = playListMap_.value(name, NULL);

    if (!playList)
        delete playList; // triggers destroyed() -> playListDestroyed() above...

    return playList;
}

PlayListAspect *PlayListController::createPlayListAspect(PlayList *playList, const QString &aspectName)
{
    QString newPrefix = playList->sourceTable() + "_" + aspectName + "_";

    if (!playListAspectMap_.contains(newPrefix))
    {
        PlayListAspect *result = new PlayListAspect(playList, aspectName);

        connect(result, SIGNAL(destroyed()), this, SLOT(playListAspectDestroyed()));

        playListAspects_.append(result);
        playListAspectMap_.insert(newPrefix, result);
        return result;
    }
    else
        return NULL;
}

bool PlayListController::deletePlayListAspect(const QString &prefix)
{
    PlayListAspect *playListAspect = playListAspectMap_.value(prefix, NULL);

    if (!playListAspect)
        delete playListAspect; // triggers destroyed() -> playListAspectDestroyed() above...

    return playListAspect;
}

bool PlayListController::deletePlayListAspect(PlayList *playList, const QString &aspectName)
{
    QString prefix = playList->sourceTable() + "_" + aspectName + "_";
    return deletePlayListAspect(prefix);
}

PlayListAspect *PlayListController::playListAspect(const QString &playListName, const QString &aspectName)
{
    QString prefix = playListName + "_" + aspectName + "_";
    return playListAspect(prefix);
}

PlayListAspect *PlayListController::playListAspect(PlayList *playList, const QString &aspectName)
{
    QString prefix = playList->sourceTable() + "_" + aspectName + "_";
    return playListAspect(prefix);
}

void PlayListController::loadStates()
{
    for (int i = 0; i < playLists_.count(); ++i)
        playLists_.at(i)->loadFromDatabase();

    for (int i = 0; i < playListAspects_.count(); ++i)
        playListAspects_.at(i)->loadFromDatabase();
}

void PlayListController::saveStates()
{
    for (int i = 0; i < playLists_.count(); ++i)
        playLists_.at(i)->saveToDatabase();

    for (int i = 0; i < playListAspects_.count(); ++i)
        playListAspects_.at(i)->saveToDatabase();
}
